@@ -1,9 +1,9 @@
-health_status = {}
+local health_status = {}
 if obj.status ~= nil then
   if obj.status.conditions ~= nil then
-    numTrue = 0
-    numFalse = 0
-    msg = ""
+    local numTrue = 0
+    local numFalse = 0
+    local msg = ""
     for i, condition in pairs(obj.status.conditions) do
       msg = msg .. i .. ": " .. condition.type .. " | " .. condition.status .. "\n"
       if condition.type == "Ready" and condition.status == "True" then
@@ -14,6 +14,8 @@ if obj.status ~= nil then
         numTrue = numTrue + 1
       elseif condition.type == "DeploymentsAvailable" and condition.status == "True" then
         numTrue = numTrue + 1
+      elseif condition.type == "Ready" and condition.status == "False" then
+        numFalse = numFalse + 1
       elseif condition.type == "DeploymentsAvailable" and condition.status == "False" then
         numFalse = numFalse + 1
       elseif condition.status == "Unknown" then
