@@ -4,17 +4,17 @@ import (
 	"context"
 	"fmt"
 
-	"github.com/ghodss/yaml"
 	"github.com/golang/protobuf/ptypes/empty"
+	"sigs.k8s.io/yaml"
 
-	"github.com/argoproj/argo-cd/v2/reposerver/apiclient"
-	ioutil "github.com/argoproj/argo-cd/v2/util/io"
+	"github.com/argoproj/argo-cd/v3/reposerver/apiclient"
+	utilio "github.com/argoproj/argo-cd/v3/util/io"
 
-	sessionmgr "github.com/argoproj/argo-cd/v2/util/session"
+	sessionmgr "github.com/argoproj/argo-cd/v3/util/session"
 
-	settingspkg "github.com/argoproj/argo-cd/v2/pkg/apiclient/settings"
-	"github.com/argoproj/argo-cd/v2/pkg/apis/application/v1alpha1"
-	"github.com/argoproj/argo-cd/v2/util/settings"
+	settingspkg "github.com/argoproj/argo-cd/v3/pkg/apiclient/settings"
+	"github.com/argoproj/argo-cd/v3/pkg/apis/application/v1alpha1"
+	"github.com/argoproj/argo-cd/v3/util/settings"
 )
 
 // Server provides a Settings service
@@ -24,6 +24,7 @@ type Server struct {
 	authenticator             Authenticator
 	disableAuth               bool
 	appsInAnyNamespaceEnabled bool
+	hydratorEnabled           bool
 }
 
 type Authenticator interface {
@@ -31,12 +32,12 @@ type Authenticator interface {
 }
 
 // NewServer returns a new instance of the Settings service
-func NewServer(mgr *settings.SettingsManager, repoClient apiclient.Clientset, authenticator Authenticator, disableAuth, appsInAnyNamespaceEnabled bool) *Server {
-	return &Server{mgr: mgr, repoClient: repoClient, authenticator: authenticator, disableAuth: disableAuth, appsInAnyNamespaceEnabled: appsInAnyNamespaceEnabled}
+func NewServer(mgr *settings.SettingsManager, repoClient apiclient.Clientset, authenticator Authenticator, disableAuth, appsInAnyNamespaceEnabled bool, hydratorEnabled bool) *Server {
+	return &Server{mgr: mgr, repoClient: repoClient, authenticator: authenticator, disableAuth: disableAuth, appsInAnyNamespaceEnabled: appsInAnyNamespaceEnabled, hydratorEnabled: hydratorEnabled}
 }
 
 // Get returns Argo CD settings
-func (s *Server) Get(ctx context.Context, q *settingspkg.SettingsQuery) (*settingspkg.Settings, error) {
+func (s *Server) Get(ctx context.Context, _ *settingspkg.SettingsQuery) (*settingspkg.Settings, error) {
 	resourceOverrides, err := s.mgr.GetResourceOverrides()
 	if err != nil {
 		return nil, err
@@ -62,10 +63,6 @@ func (s *Server) Get(ctx context.Context, q *settingspkg.SettingsQuery) (*settin
 	if err != nil {
 		return nil, err
 	}
-	plugins, err := s.plugins(ctx, false)
-	if err != nil {
-		return nil, err
-	}
 	userLoginsDisabled := true
 	accounts, err := s.mgr.GetAccounts()
 	if err != nil {
@@ -94,6 +91,7 @@ func (s *Server) Get(ctx context.Context, q *settingspkg.SettingsQuery) (*settin
 
 	set := settingspkg.Settings{
 		URL:                argoCDSettings.URL,
+		AdditionalURLs:     argoCDSettings.AdditionalURLs,
 		AppLabelKey:        appInstanceLabelKey,
 		ResourceOverrides:  overrides,
 		StatusBadgeEnabled: argoCDSettings.StatusBadgeEnabled,
@@ -110,32 +108,26 @@ func (s *Server) Get(ctx context.Context, q *settingspkg.SettingsQuery) (*settin
 			ChatText:   help.ChatText,
 			BinaryUrls: help.BinaryURLs,
 		},
-		Plugins:                   plugins,
 		UserLoginsDisabled:        userLoginsDisabled,
 		KustomizeVersions:         kustomizeVersions,
 		UiCssURL:                  argoCDSettings.UiCssURL,
-		PasswordPattern:           argoCDSettings.PasswordPattern,
 		TrackingMethod:            trackingMethod,
 		ExecEnabled:               argoCDSettings.ExecEnabled,
 		AppsInAnyNamespaceEnabled: s.appsInAnyNamespaceEnabled,
+		ImpersonationEnabled:      argoCDSettings.ImpersonationEnabled,
+		HydratorEnabled:           s.hydratorEnabled,
 	}
 
 	if sessionmgr.LoggedIn(ctx) || s.disableAuth {
-		configManagementPlugins, err := s.mgr.GetConfigManagementPlugins()
-		if err != nil {
-			return nil, err
-		}
-		tools := make([]*v1alpha1.ConfigManagementPlugin, len(configManagementPlugins))
-		for i := range configManagementPlugins {
-			tools[i] = &configManagementPlugins[i]
-		}
-		set.ConfigManagementPlugins = tools
 		set.UiBannerContent = argoCDSettings.UiBannerContent
 		set.UiBannerURL = argoCDSettings.UiBannerURL
 		set.UiBannerPermanent = argoCDSettings.UiBannerPermanent
 		set.UiBannerPosition = argoCDSettings.UiBannerPosition
 		set.ControllerNamespace = s.mgr.GetNamespace()
 	}
+	if sessionmgr.LoggedIn(ctx) {
+		set.PasswordPattern = argoCDSettings.PasswordPattern
+	}
 	if argoCDSettings.DexConfig != "" {
 		var cfg settingspkg.DexConfig
 		err = yaml.Unmarshal([]byte(argoCDSettings.DexConfig), &cfg)
@@ -145,11 +137,12 @@ func (s *Server) Get(ctx context.Context, q *settingspkg.SettingsQuery) (*settin
 	}
 	if oidcConfig := argoCDSettings.OIDCConfig(); oidcConfig != nil {
 		set.OIDCConfig = &settingspkg.OIDCConfig{
-			Name:        oidcConfig.Name,
-			Issuer:      oidcConfig.Issuer,
-			ClientID:    oidcConfig.ClientID,
-			CLIClientID: oidcConfig.CLIClientID,
-			Scopes:      oidcConfig.RequestedScopes,
+			Name:                     oidcConfig.Name,
+			Issuer:                   oidcConfig.Issuer,
+			ClientID:                 oidcConfig.ClientID,
+			CLIClientID:              oidcConfig.CLIClientID,
+			Scopes:                   oidcConfig.RequestedScopes,
+			EnablePKCEAuthentication: oidcConfig.EnablePKCEAuthentication,
 		}
 		if len(argoCDSettings.OIDCConfig().RequestedIDTokenClaims) > 0 {
 			set.OIDCConfig.IDTokenClaims = argoCDSettings.OIDCConfig().RequestedIDTokenClaims
@@ -159,40 +152,30 @@ func (s *Server) Get(ctx context.Context, q *settingspkg.SettingsQuery) (*settin
 }
 
 // GetPlugins returns a list of plugins
-func (s *Server) GetPlugins(ctx context.Context, q *settingspkg.SettingsQuery) (*settingspkg.SettingsPluginsResponse, error) {
-	plugins, err := s.plugins(ctx, true)
+func (s *Server) GetPlugins(ctx context.Context, _ *settingspkg.SettingsQuery) (*settingspkg.SettingsPluginsResponse, error) {
+	plugins, err := s.plugins(ctx)
 	if err != nil {
 		return nil, err
 	}
 	return &settingspkg.SettingsPluginsResponse{Plugins: plugins}, nil
 }
 
-func (s *Server) plugins(ctx context.Context, includeV2Plugins bool) ([]*settingspkg.Plugin, error) {
-	in, err := s.mgr.GetConfigManagementPlugins()
+func (s *Server) plugins(ctx context.Context) ([]*settingspkg.Plugin, error) {
+	closer, client, err := s.repoClient.NewRepoServerClient()
 	if err != nil {
-		return nil, err
+		return nil, fmt.Errorf("error creating repo server client: %w", err)
 	}
+	defer utilio.Close(closer)
+
+	pluginList, err := client.ListPlugins(ctx, &empty.Empty{})
+	if err != nil {
+		return nil, fmt.Errorf("failed to list sidecar plugins from reposerver: %w", err)
+	}
+
 	var out []*settingspkg.Plugin
-	for _, p := range in {
-		out = append(out, &settingspkg.Plugin{Name: p.Name})
-	}
-
-	if includeV2Plugins {
-		closer, client, err := s.repoClient.NewRepoServerClient()
-		if err != nil {
-			return nil, fmt.Errorf("error creating repo server client: %w", err)
-		}
-		defer ioutil.Close(closer)
-
-		pluginList, err := client.ListPlugins(ctx, &empty.Empty{})
-		if err != nil {
-			return nil, fmt.Errorf("failed to list sidecar plugins from reposerver: %w", err)
-		}
-
-		if pluginList != nil && len(pluginList.Items) > 0 {
-			for _, p := range pluginList.Items {
-				out = append(out, &settingspkg.Plugin{Name: p.Name})
-			}
+	if pluginList != nil && len(pluginList.Items) > 0 {
+		for _, p := range pluginList.Items {
+			out = append(out, &settingspkg.Plugin{Name: p.Name})
 		}
 	}
 
