@@ -37,7 +37,7 @@ export const ApplicationsTable = (props: {
         keys: Key.ENTER,
         action: () => {
             if (selectedApp > -1) {
-                ctxh.navigation.goto(`/applications/${props.applications[selectedApp].metadata.name}`);
+                ctxh.navigation.goto(AppUtils.getAppUrl(props.applications[selectedApp]));
                 return true;
             }
             return false;
@@ -57,9 +57,7 @@ export const ApplicationsTable = (props: {
                                         key={AppUtils.appInstanceName(app)}
                                         className={`argo-table-list__row
                 applications-list__entry applications-list__entry--health-${app.status.health.status} ${selectedApp === i ? 'applications-tiles__selected' : ''}`}>
-                                        <div
-                                            className={`row applications-list__table-row`}
-                                            onClick={e => ctx.navigation.goto(`/applications/${app.metadata.namespace}/${app.metadata.name}`, {}, {event: e})}>
+                                        <div className={`row applications-list__table-row`} onClick={e => ctx.navigation.goto(AppUtils.getAppUrl(app), {}, {event: e})}>
                                             <div className='columns small-4'>
                                                 <div className='row'>
                                                     <div className=' columns small-2'>
@@ -140,9 +138,21 @@ export const ApplicationsTable = (props: {
                                                         </button>
                                                     )}
                                                     items={[
-                                                        {title: 'Sync', action: () => props.syncApplication(app.metadata.name, app.metadata.namespace)},
-                                                        {title: 'Refresh', action: () => props.refreshApplication(app.metadata.name, app.metadata.namespace)},
-                                                        {title: 'Delete', action: () => props.deleteApplication(app.metadata.name, app.metadata.namespace)}
+                                                        {
+                                                            title: 'Sync',
+                                                            iconClassName: 'fa fa-fw fa-sync',
+                                                            action: () => props.syncApplication(app.metadata.name, app.metadata.namespace)
+                                                        },
+                                                        {
+                                                            title: 'Refresh',
+                                                            iconClassName: 'fa fa-fw fa-redo',
+                                                            action: () => props.refreshApplication(app.metadata.name, app.metadata.namespace)
+                                                        },
+                                                        {
+                                                            title: 'Delete',
+                                                            iconClassName: 'fa fa-fw fa-times-circle',
+                                                            action: () => props.deleteApplication(app.metadata.name, app.metadata.namespace)
+                                                        }
                                                     ]}
                                                 />
                                             </div>
