@@ -3,6 +3,7 @@ package env
 import (
 	"fmt"
 	"math"
+	"strconv"
 	"testing"
 	"time"
 
@@ -12,8 +13,8 @@ import (
 
 func TestParseNumFromEnv(t *testing.T) {
 	const envKey = "SOMEKEY"
-	const min = math.MinInt + 1
-	const max = math.MaxInt - 1
+	const minimum = math.MinInt + 1
+	const maximum = math.MaxInt - 1
 	const def = 10
 	testCases := []struct {
 		name     string
@@ -23,17 +24,17 @@ func TestParseNumFromEnv(t *testing.T) {
 		{"Valid positive number", "200", 200},
 		{"Valid negative number", "-200", -200},
 		{"Invalid number", "abc", def},
-		{"Equals minimum", fmt.Sprintf("%d", math.MinInt+1), min},
-		{"Equals maximum", fmt.Sprintf("%d", math.MaxInt-1), max},
-		{"Less than minimum", fmt.Sprintf("%d", math.MinInt), def},
-		{"Greater than maximum", fmt.Sprintf("%d", math.MaxInt), def},
+		{"Equals minimum", strconv.Itoa(math.MinInt + 1), minimum},
+		{"Equals maximum", strconv.Itoa(math.MaxInt - 1), maximum},
+		{"Less than minimum", strconv.Itoa(math.MinInt), def},
+		{"Greater than maximum", strconv.Itoa(math.MaxInt), def},
 		{"Variable not set", "", def},
 	}
 
 	for _, tt := range testCases {
 		t.Run(tt.name, func(t *testing.T) {
 			t.Setenv(envKey, tt.env)
-			n := ParseNumFromEnv(envKey, def, min, max)
+			n := ParseNumFromEnv(envKey, def, minimum, maximum)
 			assert.Equal(t, tt.expected, n)
 		})
 	}
@@ -41,8 +42,8 @@ func TestParseNumFromEnv(t *testing.T) {
 
 func TestParseFloatFromEnv(t *testing.T) {
 	const envKey = "SOMEKEY"
-	var min float32 = -1000.5
-	var max float32 = 1000.5
+	var minimum float32 = -1000.5
+	var maximum float32 = 1000.5
 	const def float32 = 10.5
 	testCases := []struct {
 		name     string
@@ -53,26 +54,26 @@ func TestParseFloatFromEnv(t *testing.T) {
 		{"Valid negative float", "-2.0", -2.0},
 		{"Valid integer as float", "2", 2.0},
 		{"Text as invalid float", "abc", def},
-		{"Equals maximum", fmt.Sprintf("%v", max), max},
-		{"Equals minimum", fmt.Sprintf("%v", min), min},
-		{"Greater than maximum", fmt.Sprintf("%f", max+1), def},
-		{"Lesser than minimum", fmt.Sprintf("%f", min-1), def},
+		{"Equals maximum", fmt.Sprintf("%v", maximum), maximum},
+		{"Equals minimum", fmt.Sprintf("%v", minimum), minimum},
+		{"Greater than maximum", fmt.Sprintf("%f", maximum+1), def},
+		{"Lesser than minimum", fmt.Sprintf("%f", minimum-1), def},
 		{"Environment not set at", "", def},
 	}
 
 	for _, tt := range testCases {
 		t.Run(tt.name, func(t *testing.T) {
 			t.Setenv(envKey, tt.env)
-			f := ParseFloatFromEnv(envKey, def, min, max)
-			assert.Equal(t, tt.expected, f)
+			f := ParseFloatFromEnv(envKey, def, minimum, maximum)
+			assert.InEpsilon(t, tt.expected, f, 0.0001)
 		})
 	}
 }
 
 func TestParseInt64FromEnv(t *testing.T) {
 	const envKey = "SOMEKEY"
-	const min int64 = 1
-	const max int64 = math.MaxInt64 - 1
+	const minimum int64 = 1
+	const maximum int64 = math.MaxInt64 - 1
 	const def int64 = 10
 	testCases := []struct {
 		name     string
@@ -81,17 +82,17 @@ func TestParseInt64FromEnv(t *testing.T) {
 	}{
 		{"Valid int64", "200", 200},
 		{"Text as invalid int64", "abc", def},
-		{"Equals maximum", fmt.Sprintf("%d", max), max},
-		{"Equals minimum", fmt.Sprintf("%d", min), min},
-		{"Greater than maximum", fmt.Sprintf("%d", max+1), def},
-		{"Less than minimum", fmt.Sprintf("%d", min-1), def},
+		{"Equals maximum", strconv.FormatInt(maximum, 10), maximum},
+		{"Equals minimum", strconv.FormatInt(minimum, 10), minimum},
+		{"Greater than maximum", strconv.FormatInt(maximum+1, 10), def},
+		{"Less than minimum", strconv.FormatInt(minimum-1, 10), def},
 		{"Environment not set", "", def},
 	}
 
 	for _, tt := range testCases {
 		t.Run(tt.name, func(t *testing.T) {
 			t.Setenv(envKey, tt.env)
-			n := ParseInt64FromEnv(envKey, def, min, max)
+			n := ParseInt64FromEnv(envKey, def, minimum, maximum)
 			assert.Equal(t, tt.expected, n)
 		})
 	}
@@ -100,8 +101,8 @@ func TestParseInt64FromEnv(t *testing.T) {
 func TestParseDurationFromEnv(t *testing.T) {
 	envKey := "SOMEKEY"
 	def := 3 * time.Second
-	min := 2 * time.Second
-	max := 5 * time.Second
+	minimum := 2 * time.Second
+	maximum := 5 * time.Second
 
 	testCases := []struct {
 		name     string
@@ -114,6 +115,10 @@ func TestParseDurationFromEnv(t *testing.T) {
 		name:     "ValidValueSet",
 		env:      "2s",
 		expected: time.Second * 2,
+	}, {
+		name:     "ValidValueSetMs",
+		env:      "2500ms",
+		expected: time.Millisecond * 2500,
 	}, {
 		name:     "MoreThanMaxSet",
 		env:      "6s",
@@ -131,7 +136,91 @@ func TestParseDurationFromEnv(t *testing.T) {
 	for _, tc := range testCases {
 		t.Run(tc.name, func(t *testing.T) {
 			t.Setenv(envKey, tc.env)
-			val := ParseDurationFromEnv(envKey, def, min, max)
+			val := ParseDurationFromEnv(envKey, def, minimum, maximum)
+			assert.Equal(t, tc.expected, val)
+		})
+	}
+}
+
+func TestParseDurationFromEnvEdgeCases(t *testing.T) {
+	envKey := "SOME_ENV_KEY"
+	def := 3 * time.Minute
+	minimum := 1 * time.Second
+	maximum := 2160 * time.Hour // 3 months
+
+	testCases := []struct {
+		name     string
+		env      string
+		expected time.Duration
+	}{{
+		name:     "EnvNotSet",
+		expected: def,
+	}, {
+		name:     "Durations defined as days are valid",
+		env:      "12d",
+		expected: time.Hour * 24 * 12,
+	}, {
+		name:     "Negative durations should fail parsing and use the default value",
+		env:      "-1h",
+		expected: def,
+	}, {
+		name:     "Negative day durations should fail parsing and use the default value",
+		env:      "-12d",
+		expected: def,
+	}, {
+		name:     "Scientific notation should fail parsing and use the default value",
+		env:      "1e3s",
+		expected: def,
+	}, {
+		name:     "Durations with a leading zero are considered valid and parsed as decimal notation",
+		env:      "0755s",
+		expected: time.Second * 755,
+	}, {
+		name:     "Durations with many leading zeroes are considered valid and parsed as decimal notation",
+		env:      "000083m",
+		expected: time.Minute * 83,
+	}, {
+		name:     "Decimal Durations should not fail parsing",
+		env:      "30.5m",
+		expected: time.Minute*30 + time.Second*30,
+	}, {
+		name:     "Decimal Day Durations should fail parsing and use the default value",
+		env:      "30.5d",
+		expected: def,
+	}, {
+		name:     "Fraction Durations should fail parsing and use the default value",
+		env:      "1/2h",
+		expected: def,
+	}, {
+		name:     "Durations without a time unit should fail parsing and use the default value",
+		env:      "15",
+		expected: def,
+	}, {
+		name:     "Durations with a trailing symbol should fail parsing and use the default value",
+		env:      "+12d",
+		expected: def,
+	}, {
+		name:     "Leading space Duration should fail parsing use the default value",
+		env:      " 2h",
+		expected: def,
+	}, {
+		name:     "Trailing space Duration should fail parsing use the default value",
+		env:      "6m ",
+		expected: def,
+	}, {
+		name:     "Empty Duration should fail parsing use the default value",
+		env:      "",
+		expected: def,
+	}, {
+		name:     "Whitespace Duration should fail parsing and use the default value",
+		env:      "    ",
+		expected: def,
+	}}
+
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			t.Setenv(envKey, tc.env)
+			val := ParseDurationFromEnv(envKey, def, minimum, maximum)
 			assert.Equal(t, tc.expected, val)
 		})
 	}
